from pathlib import Path
from typing import Annotated, Optional

from pydantic import BaseModel, Field


class Environment(BaseModel):
    google_service_account_email: str
    google_quota_project_id: Annotated[Optional[str], Field(default=None)]
    aws_role_arn: Annotated[str, Field(pattern="^arn:aws:iam::[0-9]+:role/[A-Za-z0-9_-]+$")]
    aws_region: Annotated[str, Field(default="eu-west-2", pattern="^[a-z0-9-]+$")]


class Config(BaseModel):
    environments: Annotated[dict[str, Environment], Field(default_factory=dict)]


class ConfigNotFoundError(RuntimeError):
    def __init__(self):
        super().__init__("No configuration file found")


def find_config(config_file: Optional[str | Path] = None) -> Path:
    """
    Look for a configuration file starting from the current directory and walking upwards until we
    reach a file system boundary.

    Arguments:
        config_file: override the search process and specify the location directly

    Returns:
        A Path pointing to the location of the discovered configuration file.

    Raises:
        ConfigNotFound: If no configuration file could be found.
    """
    # If we were given a path, simply parse and convert it to an absolute location.
    if config_file is not None:
        return Path(config_file).absolute()

    # Otherwise, search for the config.
    dir = Path.cwd()
    while dir != dir.root:
        config_path = dir / ".aws-helper.yaml"
        if config_path.is_file():
            return config_path.absolute()
        if dir.is_mount():
            break
        dir = dir.parent

    raise ConfigNotFoundError()
